from django.test import TestCase

from entity_event_slack.models import SlackMedium


class SlackMediumManagerTest(TestCase):
    def test_create_slack_medium_no_existing(self):
        m = SlackMedium.objects.create_slack_medium(channel='#slack-channel', api_token='api_token')

        self.assertEquals(m.channel, '#slack-channel')
        self.assertEquals(m.api_token, 'api_token')

    def test_create_slack_medium_w_existing(self):
        m = SlackMedium.objects.create_slack_medium(channel='#slack-channel', api_token='api_token')
        m = SlackMedium.objects.create_slack_medium(channel='new channel', api_token='new api token')

        # Medium should not have been updated on second creation
        self.assertEquals(m.channel, '#slack-channel')
        self.assertEquals(m.api_token, 'api_token')



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'SlackMedium.icon_url'
        db.add_column(u'entity_event_slack_slackmedium', 'icon_url',
                      self.gf('django.db.models.fields.TextField')(default=''),
                      keep_default=False)

        # Adding field 'SlackMedium.username'
        db.add_column(u'entity_event_slack_slackmedium', 'username',
                      self.gf('django.db.models.fields.TextField')(default=''),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'SlackMedium.icon_url'
        db.delete_column(u'entity_event_slack_slackmedium', 'icon_url')

        # Deleting field 'SlackMedium.username'
        db.delete_column(u'entity_event_slack_slackmedium', 'username')


    models = {
        u'entity_event.medium': {
            'Meta': {'object_name': 'Medium'},
            'description': ('django.db.models.fields.TextField', [], {}),
            'display_name': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'}),
            'rendering_style': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['entity_event.RenderingStyle']", 'null': 'True'})
        },
        u'entity_event.renderingstyle': {
            'Meta': {'object_name': 'RenderingStyle'},
            'display_name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '64'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'})
        },
        u'entity_event_slack.slackmedium': {
            'Meta': {'object_name': 'SlackMedium', '_ormbases': [u'entity_event.Medium']},
            'api_token': ('django.db.models.fields.TextField', [], {}),
            'channel': ('django.db.models.fields.TextField', [], {}),
            'creation_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'icon_url': ('django.db.models.fields.TextField', [], {'default': "''"}),
            u'medium_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['entity_event.Medium']", 'unique': 'True', 'primary_key': 'True'}),
            'username': ('django.db.models.fields.TextField', [], {'default': "''"})
        }
    }

    complete_apps = ['entity_event_slack']


from django.db import models
from entity_event.models import Medium

from entity_event_slack import constants


class SlackMediumManager(models.Manager):
    def get_queryset(self):
        return super(SlackMediumManager, self).get_queryset().filter(name=constants.SLACK_MEDIUM_NAME)

    def create_slack_medium(self, **kwargs):
        return self.get_or_create(name=constants.SLACK_MEDIUM_NAME, defaults=kwargs)[0]


class SlackMedium(Medium):
    """
    A medium that extends djagno entity event's Medium to include
    Slack-specific configuration. When this medium is present, events will be pushed to this
    medium using the configured api_token and channel.
    """
    # The slack API token
    api_token = models.TextField()

    # The slack channel to post events to
    channel = models.TextField()

    # The time at which this medium was created. Events before this time will never be
    # pushed
    creation_time = models.DateTimeField(auto_now_add=True)

    # The icon url to use for the bot
    icon_url = models.TextField(default='')

    # The username for this bot
    username = models.TextField(default='')

    objects = SlackMediumManager()



from celery import Task
from db_mutex import db_mutex
from manager_utils import get_or_none
import slack
import slack.chat

from entity_event_slack.models import SlackMedium


class SendUnseenSlackNotificationsTask(Task):
    """
    Converts events to emails based on the email subscriptions.
    """
    def run(self, *args, **kwargs):
        with db_mutex('push-slack-events'):
            self.run_worker(*args, **kwargs)

    def run_worker(self, *args, **kwargs):
        send_unseen_slack_notifications()


def send_unseen_slack_notifications():
    """
    Pushes rendered notifications to slack.
    """
    slack_medium = get_or_none(SlackMedium.objects)

    if slack_medium:
        slack.api_token = slack_medium.api_token

        # Gather all events while marking them as seen. Only events after the creation of the slack
        # medium are gathered.
        events = [e for e, targets in slack_medium.events_targets(
            seen=False, mark_seen=True, start_time=slack_medium.creation_time)]

        # Render the events. If rendering fails for any event, no slack notifications will
        # be sent.
        rendered_events = slack_medium.render(events)

        # Try to post them to the configured slack channel. If the post fails, all events will
        # be marked as seen and never sent to slack. This is a safety measure to ensure we dont
        # send duplicate events to slack
        msg = '\n'.join([html for (txt, html) in rendered_events.values()])

        if msg:
            slack.chat.post_message(
                slack_medium.channel, msg, username=slack_medium.username, icon_url=slack_medium.icon_url)



from datetime import datetime
from django.test import TestCase
from django_dynamic_fixture import G
from entity_event.models import RenderingStyle, ContextRenderer, Source, Subscription, Event, EventSeen
from freezegun import freeze_time
from mock import patch, call
import slack

from entity_event_slack.models import SlackMedium
from entity_event_slack.tasks import send_unseen_slack_notifications, SendUnseenSlackNotificationsTask


class TestRunWorker(TestCase):
    @patch('entity_event_slack.tasks.send_unseen_slack_notifications', spec_set=True)
    def test_run_worker(self, mock_send_unseen_slack_notifications):
        SendUnseenSlackNotificationsTask().run()
        mock_send_unseen_slack_notifications.assert_called_once_with()


class TestSendUnseenNotifications(TestCase):
    @patch('entity_event_slack.tasks.slack.chat.post_message', spec_set=True)
    def test_no_slack_medium(self, mock_post_message):
        send_unseen_slack_notifications()
        self.assertFalse(mock_post_message.called)

    @patch('entity_event_slack.tasks.slack.chat.post_message', spec_set=True)
    def test_no_entity_event_setup(self, mock_post_message):
        G(SlackMedium, name='slack', api_token='api_token')
        send_unseen_slack_notifications()
        self.assertFalse(mock_post_message.called)
        self.assertEquals(slack.api_token, 'api_token')

    @freeze_time(datetime(2015, 1, 1))
    @patch('entity_event_slack.tasks.slack.chat.post_message', spec_set=True)
    def test_w_basic_entity_event_setup(self, mock_post_message):
        style = G(RenderingStyle)
        source = G(Source)
        slack_medium = G(
            SlackMedium, channel='channel', name='slack', api_token='api_token', rendering_style=style,
            icon_url='icon_url', username='user')
        G(ContextRenderer, rendering_style=style, source=source, html_template='You did it!')
        G(Subscription, medium=slack_medium, source=source, only_following=False)

        G(Event, time=datetime(2015, 1, 2), context={}, source=source)

        send_unseen_slack_notifications()

        self.assertEquals(slack_medium.creation_time, datetime(2015, 1, 1))
        self.assertEquals(mock_post_message.call_args_list, [
            call('channel', 'You did it!', icon_url='icon_url', username='user')
        ])
        self.assertEquals(slack.api_token, 'api_token')
        self.assertEquals(EventSeen.objects.count(), 1)

    @freeze_time(datetime(2015, 1, 1))
    @patch('entity_event_slack.tasks.slack.chat.post_message', spec_set=True)
    def test_w_basic_entity_event_setup_preexisting_event(self, mock_post_message):
        style = G(RenderingStyle)
        source = G(Source)
        slack_medium = G(
            SlackMedium, channel='channel', name='slack', api_token='api_token', rendering_style=style,
            icon_url='icon_url', username='user')
        G(ContextRenderer, rendering_style=style, source=source, html_template='You did it!')
        G(Subscription, medium=slack_medium, source=source, only_following=False)

        G(Event, time=datetime(2014, 1, 2), context={}, source=source)

        send_unseen_slack_notifications()

        self.assertEquals(slack_medium.creation_time, datetime(2015, 1, 1))
        self.assertFalse(mock_post_message.called)
        self.assertEquals(slack.api_token, 'api_token')
        self.assertEquals(EventSeen.objects.count(), 0)

    @freeze_time(datetime(2015, 1, 1))
    @patch('entity_event_slack.tasks.slack.chat.post_message', spec_set=True)
    def test_w_basic_entity_multiple_event_setup(self, mock_post_message):
        style = G(RenderingStyle)
        source = G(Source)
        slack_medium = G(
            SlackMedium, icon_url='icon_url', channel='channel', name='slack', api_token='api_token',
            rendering_style=style, username='user')
        G(ContextRenderer, rendering_style=style, source=source, html_template='You did it!')
        G(Subscription, medium=slack_medium, source=source, only_following=False)

        G(Event, time=datetime(2015, 1, 2), context={}, source=source)
        G(Event, time=datetime(2015, 1, 2), context={}, source=source)

        send_unseen_slack_notifications()

        self.assertEquals(slack_medium.creation_time, datetime(2015, 1, 1))
        self.assertEquals(mock_post_message.call_args_list, [
            call('channel', 'You did it!\nYou did it!', icon_url='icon_url', username='user'),
        ])
        self.assertEquals(slack.api_token, 'api_token')
        self.assertEquals(EventSeen.objects.count(), 2)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'SlackMedium'
        db.create_table(u'entity_event_slack_slackmedium', (
            (u'medium_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['entity_event.Medium'], unique=True, primary_key=True)),
            ('api_token', self.gf('django.db.models.fields.TextField')()),
            ('channel', self.gf('django.db.models.fields.TextField')()),
            ('creation_time', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'entity_event_slack', ['SlackMedium'])


    def backwards(self, orm):
        # Deleting model 'SlackMedium'
        db.delete_table(u'entity_event_slack_slackmedium')


    models = {
        u'entity_event.medium': {
            'Meta': {'object_name': 'Medium'},
            'description': ('django.db.models.fields.TextField', [], {}),
            'display_name': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'}),
            'rendering_style': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['entity_event.RenderingStyle']", 'null': 'True'})
        },
        u'entity_event.renderingstyle': {
            'Meta': {'object_name': 'RenderingStyle'},
            'display_name': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '64'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'})
        },
        u'entity_event_slack.slackmedium': {
            'Meta': {'object_name': 'SlackMedium', '_ormbases': [u'entity_event.Medium']},
            'api_token': ('django.db.models.fields.TextField', [], {}),
            'channel': ('django.db.models.fields.TextField', [], {}),
            'creation_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'medium_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['entity_event.Medium']", 'unique': 'True', 'primary_key': 'True'})
        }
    }

    complete_apps = ['entity_event_slack']